@@ -1,7 +1,7 @@
 /*
- * irc.c - IRC (Internet Relay Chat) plugin for WeeChat
+ * SPDX-FileCopyrightText: 2003-2025 Sébastien Helleu <flashcode@flashtux.org>
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -19,6 +19,8 @@
  * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+/* IRC (Internet Relay Chat) plugin for WeeChat */
+
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -37,6 +39,7 @@
 #include "irc-ignore.h"
 #include "irc-info.h"
 #include "irc-input.h"
+#include "irc-list.h"
 #include "irc-nick.h"
 #include "irc-notify.h"
 #include "irc-protocol.h"
@@ -60,7 +63,6 @@ struct t_weechat_plugin *weechat_irc_plugin = NULL;
 struct t_hook *irc_hook_timer = NULL;
 
 int irc_signal_quit_received = 0;      /* signal "quit" received?           */
-int irc_signal_upgrade_received = 0;   /* signal "upgrade" received?        */
 
 
 /*
@@ -120,12 +122,17 @@ irc_signal_upgrade_cb (const void *pointer, void *data,
          * save session with a disconnected state in servers and a scheduled
          * reconnection
          */
-        irc_upgrade_save (1);
+        if (!irc_upgrade_save (1))
+        {
+            weechat_printf (
+                NULL,
+                _("%s%s: failed to save upgrade data"),
+                weechat_prefix ("error"), IRC_PLUGIN_NAME);
+            return WEECHAT_RC_ERROR;
+        }
         return WEECHAT_RC_OK;
     }
 
-    irc_signal_upgrade_received = 1;
-
     quit = (signal_data && (strcmp (signal_data, "quit") == 0));
 
     tls_disconnected = 0;
@@ -149,6 +156,8 @@ irc_signal_upgrade_cb (const void *pointer, void *data,
                       "work for servers connected via TLS"),
                     weechat_prefix ("error"), IRC_PLUGIN_NAME);
             }
+            /* send QUIT to server, then disconnect */
+            irc_command_quit_server (ptr_server, NULL);
             irc_server_disconnect (ptr_server, 0, 0);
             /*
              * schedule reconnection: WeeChat will reconnect to this server
@@ -175,6 +184,15 @@ irc_signal_upgrade_cb (const void *pointer, void *data,
             tls_disconnected);
     }
 
+    if (!irc_upgrade_save (0))
+    {
+        weechat_printf (
+            NULL,
+            _("%s%s: failed to save upgrade data"),
+            weechat_prefix ("error"), IRC_PLUGIN_NAME);
+        return WEECHAT_RC_ERROR;
+    }
+
     return WEECHAT_RC_OK;
 }
 
@@ -190,11 +208,17 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 
     weechat_plugin = plugin;
 
+    irc_signal_quit_received = 0;
+
+    irc_color_init ();
+
     if (!irc_config_init ())
         return WEECHAT_RC_ERROR;
 
     irc_config_read ();
 
+    irc_list_init ();
+
     irc_raw_init ();
 
     irc_command_init ();
@@ -221,12 +245,16 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
                          &irc_input_send_cb, NULL, NULL);
     weechat_hook_signal ("typing_self_*",
                          &irc_typing_signal_typing_self_cb, NULL, NULL);
+    weechat_hook_signal ("window_scrolled",
+                         &irc_list_window_scrolled_cb, NULL, NULL);
 
     /* hook hsignals for redirection */
     weechat_hook_hsignal ("irc_redirect_pattern",
                           &irc_redirect_pattern_hsignal_cb, NULL, NULL);
     weechat_hook_hsignal ("irc_redirect_command",
                           &irc_redirect_command_hsignal_cb, NULL, NULL);
+    weechat_hook_hsignal ("irc_redirection_server_*_list",
+                          &irc_list_hsignal_redirect_list_cb, NULL, NULL);
 
     /* modifiers */
     weechat_hook_modifier ("irc_color_decode",
@@ -251,8 +279,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     info_auto_connect = weechat_info_get ("auto_connect", NULL);
     auto_connect = (info_auto_connect && (strcmp (info_auto_connect, "1") == 0)) ?
         1 : 0;
-    if (info_auto_connect)
-        free (info_auto_connect);
+    free (info_auto_connect);
 
     /* look at arguments */
     for (i = 0; i < argc; i++)
@@ -305,12 +332,14 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
     (void) plugin;
 
     if (irc_hook_timer)
+    {
         weechat_unhook (irc_hook_timer);
+        irc_hook_timer = NULL;
+    }
 
-    if (irc_signal_upgrade_received)
+    if (weechat_irc_plugin->unload_with_upgrade)
     {
         irc_config_write (1);
-        irc_upgrade_save (0);
     }
     else
     {
@@ -320,6 +349,8 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
 
     irc_ignore_free_all ();
 
+    irc_list_end ();
+
     irc_raw_end ();
 
     irc_server_free_all ();
