@@ -1,5 +1,7 @@
 /*
- * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
+ * SPDX-FileCopyrightText: 2003-2025 Sébastien Helleu <flashcode@flashtux.org>
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -62,6 +64,8 @@ extern void relay_weechat_msg_add_int (struct t_relay_weechat_msg *msg,
                                        int value);
 extern void relay_weechat_msg_add_long (struct t_relay_weechat_msg *msg,
                                         long value);
+extern void relay_weechat_msg_add_longlong (struct t_relay_weechat_msg *msg,
+                                            long long value);
 extern void relay_weechat_msg_add_string (struct t_relay_weechat_msg *msg,
                                           const char *string);
 extern void relay_weechat_msg_add_buffer (struct t_relay_weechat_msg *msg,
