@@ -1,9 +1,9 @@
 /*
- * spell.c - spell checker plugin for WeeChat
+ * SPDX-FileCopyrightText: 2006 Emmanuel Bouthenot <kolter@openics.org>
+ * SPDX-FileCopyrightText: 2006-2025 Sébastien Helleu <flashcode@flashtux.org>
+ * SPDX-FileCopyrightText: 2012 Nils Görs <weechatter@arcor.de>
  *
- * Copyright (C) 2006 Emmanuel Bouthenot <kolter@openics.org>
- * Copyright (C) 2006-2024 Sébastien Helleu <flashcode@flashtux.org>
- * Copyright (C) 2012 Nils Görs <weechatter@arcor.de>
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -21,6 +21,8 @@
  * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+/* Spell checker plugin for WeeChat */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -58,7 +60,7 @@ char *spell_nick_completer = NULL;
 int spell_len_nick_completer = 0;
 
 #ifdef USE_ENCHANT
-EnchantBroker *broker = NULL;
+EnchantBroker *spell_enchant_broker = NULL;
 #endif /* USE_ENCHANT */
 
 /*
@@ -190,7 +192,7 @@ char *spell_url_prefix[] =
  */
 
 void
-spell_warning_aspell_config ()
+spell_warning_aspell_config (void)
 {
     char *aspell_filename, *spell_filename;
 
@@ -216,10 +218,8 @@ spell_warning_aspell_config ()
                         spell_filename);
     }
 
-    if (aspell_filename)
-        free (aspell_filename);
-    if (spell_filename)
-        free (spell_filename);
+    free (aspell_filename);
+    free (spell_filename);
 }
 
 /*
@@ -233,7 +233,6 @@ spell_build_option_name (struct t_gui_buffer *buffer)
 {
     const char *plugin_name, *name;
     char *option_name;
-    int length;
 
     if (!buffer)
         return NULL;
@@ -241,12 +240,7 @@ spell_build_option_name (struct t_gui_buffer *buffer)
     plugin_name = weechat_buffer_get_string (buffer, "plugin");
     name = weechat_buffer_get_string (buffer, "name");
 
-    length = strlen (plugin_name) + 1 + strlen (name) + 1;
-    option_name = malloc (length);
-    if (!option_name)
-        return NULL;
-
-    snprintf (option_name, length, "%s.%s", plugin_name, name);
+    weechat_asprintf (&option_name, "%s.%s", plugin_name, name);
 
     return option_name;
 }
@@ -843,7 +837,7 @@ spell_modifier_cb (const void *pointer, void *data,
     }
 
     current_pos = 0;
-    while (ptr_string[0])
+    while (ptr_string && ptr_string[0])
     {
         ptr_string_orig = NULL;
 
@@ -894,7 +888,7 @@ spell_modifier_cb (const void *pointer, void *data,
                 word_end_pos_valid = word_end_pos;
             }
             ptr_end = (char *)weechat_utf8_next_char (ptr_end);
-            if (!ptr_end[0])
+            if (!ptr_end || !ptr_end[0])
                 break;
             code_point = weechat_utf8_char_int (ptr_end);
         }
@@ -915,7 +909,7 @@ spell_modifier_cb (const void *pointer, void *data,
                 while (!iswspace (code_point))
                 {
                     ptr_end = (char *)weechat_utf8_next_char (ptr_end);
-                    if (!ptr_end[0])
+                    if (!ptr_end || !ptr_end[0])
                         break;
                     code_point = weechat_utf8_char_int (ptr_end);
                 }
@@ -938,8 +932,7 @@ spell_modifier_cb (const void *pointer, void *data,
                      * the beginning of this word, save the word (we will
                      * look for suggestions after this loop)
                      */
-                    if (misspelled_word)
-                        free (misspelled_word);
+                    free (misspelled_word);
                     misspelled_word = strdup (ptr_string);
                 }
             }
@@ -995,14 +988,13 @@ spell_modifier_cb (const void *pointer, void *data,
                                                  misspelled_word);
             if (suggestions)
             {
-                length = strlen (misspelled_word) + 1 /* ":" */
-                    + strlen (suggestions) + 1;
-                word_and_suggestions = malloc (length);
-                if (word_and_suggestions)
+                if (weechat_asprintf (&word_and_suggestions,
+                                      "%s:%s",
+                                      misspelled_word,
+                                      suggestions) >= 0)
                 {
-                    snprintf (word_and_suggestions, length, "%s:%s",
-                              misspelled_word, suggestions);
-                    weechat_buffer_set (buffer, "localvar_set_spell_suggest",
+                    weechat_buffer_set (buffer,
+                                        "localvar_set_spell_suggest",
                                         word_and_suggestions);
                     free (word_and_suggestions);
                 }
@@ -1021,9 +1013,7 @@ spell_modifier_cb (const void *pointer, void *data,
             }
         }
 
-        if (old_misspelled_word)
-            free (old_misspelled_word);
-
+        free (old_misspelled_word);
         free (misspelled_word);
     }
     else
@@ -1046,8 +1036,7 @@ spell_modifier_cb (const void *pointer, void *data,
         (void) weechat_hook_signal_send ("spell_suggest",
                                          WEECHAT_HOOK_SIGNAL_POINTER, buffer);
     }
-    if (old_suggestions)
-        free (old_suggestions);
+    free (old_suggestions);
 
     ptr_speller_buffer->modifier_result = strdup (*result);
 
@@ -1165,8 +1154,7 @@ spell_config_change_nick_completer_cb (const void *pointer, void *data,
     (void) data;
     (void) option;
 
-    if (spell_nick_completer)
-        free (spell_nick_completer);
+    free (spell_nick_completer);
 
     spell_nick_completer = weechat_string_strip (value, 0, 1, " ");
     spell_len_nick_completer =
@@ -1194,9 +1182,14 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 
 #ifdef USE_ENCHANT
     /* acquire enchant broker */
-    broker = enchant_broker_init ();
-    if (!broker)
+    spell_enchant_broker = enchant_broker_init ();
+    if (!spell_enchant_broker)
         return WEECHAT_RC_ERROR;
+#ifdef ENCHANT_MYSPELL_DICT_DIR
+    enchant_broker_set_param(spell_enchant_broker,
+                             "enchant.myspell.dictionary.path",
+                             ENCHANT_MYSPELL_DICT_DIR);
+#endif
 #endif /* USE_ENCHANT */
 
     if (!spell_speller_init ())
@@ -1261,8 +1254,8 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
 
 #ifdef USE_ENCHANT
     /* release enchant broker */
-    enchant_broker_free (broker);
-    broker = NULL;
+    enchant_broker_free (spell_enchant_broker);
+    spell_enchant_broker = NULL;
 #endif /* USE_ENCHANT */
 
     if (spell_nick_completer)
